import Globals, Products
from OFS.ObjectManager import ObjectManager

class OrderedObjectManager(ObjectManager):
	""" Ordered ObjectManager - supports changing order of objects """
	meta_type='Ordered ObjectManager'
	
	__ac_permissions__ = (
	                     ('View', ('manage_first', 'manage_moveup', 'manage_movedown'),),
						)
			
	def manage_first(self, ids=[], REQUEST=None):
		""" Make some objects first """
		objects=self._objects
		if not ids:
			message="No objects were selected to be made first."
		elif len(ids)==len(objects):
			message="Making all objects first has no effect."
		else:
			objects=self._objects
			objects=tuple(
				filter(lambda object, ids=ids:
				   	   object['id'] in ids,
					   objects)
				+
				filter(lambda object, ids=ids:
					   object['id'] not in ids,
					   objects)
				)
			self._objects = objects
			message="Objects were rearranged as requested."
		
		if REQUEST is not None:
			return self.manage_main(self, REQUEST, manage_tabs_message=message)

	def manage_moveup(self, ids=[], REQUEST=None):
		""" Move objects down in the list """
	
		objects=self._objects
		if not ids:
			message="No object was selected to be moved."
		
		else:
			objects = self._objects
			numobjects = len(objects)
		
			if objects[0]['id'] in ids:
				message="Can't move up because first item was selected."
			else:
				for i in range(1, numobjects):
					if objects[i]['id'] in ids:
						objects = tuple(list(objects[0:i-1]) + list(objects[i:i+1]) +
								list(objects[i-1:i]) + list(objects[i+1:numobjects]))
				self._objects = objects	
				message="Object(s) moved up as requested."
			
		if REQUEST is not None:
			return self.manage_main(self, REQUEST, manage_tabs_message=message)		

	def manage_movedown(self, ids=[], REQUEST=None):
		""" Move objects down in the list, unless one of then is the last """
	
		objects=self._objects
		if not ids:
			message="No object was selected to be moved."
		else:
			objects = self._objects
			numobjects = len(objects)
		
			if objects[numobjects-1]['id'] in ids:
				message="Can't move down because last item was selected."
			else:
				iterator=[]
				
				#My silly implementation of for(i=numobjects-1;i>0;i--)  Noa
				for i in range(0, numobjects-1):
					iterator=[i]+iterator
  				for i in iterator:
					if objects[i]['id'] in ids:
						objects = tuple(list(objects[0:i]) + list(objects[i+1:i+2]) +
								list(objects[i:i+1]) + list(objects[i+2:numobjects]))
				self._objects = objects	
				message="Object(s) moved down as requested."
				
		if REQUEST is not None:
			return self.manage_main(self, REQUEST, manage_tabs_message=message)		
